@@ -2,6 +2,8 @@
 #'
 #' @templateVar fn DataFrame
 #' @template x
+#' @param all.missing [\code{logical(1)}]\cr
+#'  Are columns with only missing values allowed? Default is \code{TRUE}.
 #' @inheritParams checkMatrix
 #' @inheritParams checkList
 #' @template null.ok
@@ -13,8 +15,8 @@
 #' @examples
 #' testDataFrame(iris)
 #' testDataFrame(iris, types = c("numeric", "factor"), min.rows = 1, col.names = "named")
-checkDataFrame = function(x, types = character(0L), any.missing = TRUE, all.missing = TRUE, min.rows = NULL, min.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
-  .Call(c_check_dataframe, x, any.missing, all.missing, min.rows, min.cols, nrows, ncols, row.names, col.names, null.ok) %and%
+checkDataFrame = function(x, types = character(0L), any.missing = TRUE, all.missing = TRUE, min.rows = NULL, max.rows = NULL, min.cols = NULL, max.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
+  .Call(c_check_dataframe, x, any.missing, all.missing, min.rows, max.rows, min.cols, max.cols, nrows, ncols, row.names, col.names, null.ok) %and%
   checkListTypes(x, types)
 }
 
@@ -26,7 +28,7 @@ check_data_frame = checkDataFrame
 #' @include makeAssertion.R
 #' @template assert
 #' @rdname checkDataFrame
-assertDataFrame = makeAssertionFunction(checkDataFrame)
+assertDataFrame = makeAssertionFunction(checkDataFrame, use.namespace = FALSE)
 
 #' @export
 #' @rdname checkDataFrame
@@ -45,4 +47,4 @@ test_data_frame = testDataFrame
 #' @include makeExpectation.R
 #' @template expect
 #' @rdname checkDataFrame
-expect_data_frame = makeExpectationFunction(checkDataFrame)
+expect_data_frame = makeExpectationFunction(checkDataFrame, use.namespace = FALSE)
