@@ -59,3 +59,13 @@ test_that("factors are detected (#164)", {
   x = factor(letters)
   expect_error(assertInteger(x), "factor")
 })
+
+test_that("typed.missing", {
+  expect_true(testInteger(NA_character_))
+  expect_true(testInteger(NA_character_, typed.missing = FALSE))
+  expect_false(testInteger(NA_character_, typed.missing = TRUE))
+
+  expect_true(testInteger(character()))
+  expect_true(testInteger(character(), typed.missing = FALSE))
+  expect_false(testInteger(character(), typed.missing = TRUE))
+})
